@@ -1,8 +1,7 @@
-import pytz
-import datetime as dt
+
 import numpy as np
 import pandas as pd
-
+import datetime as dt
 import os
 import json
 
@@ -168,33 +167,109 @@ class lists:
 
 
 class time:
+
+    def __init__(self,input=dt.datetime.now(),
+                 output_type='string',
+                 output_format='%Y-%m-%dT%H:%M:%S',
+                 input_tz='Europe/Prague',
+                 output_tz="Europe/Prague"):
+        from dateutil.parser import parse
+        self.output_type=output_type
+        self.input=input
+        self.input_datetime=self.input if isinstance(self.input,dt.datetime) else parse(self.input,ignoretz=True)
+        self.input_timezone=input_tz
+        self.output_type = output_type
+        self.output_timezone = output_tz
+        self.output_format = output_format
+        self.output_datetime=self.input_datetime
+        self.output = self.output_datetime if self.output_type == 'datetime' else self.output_datetime.strftime(
+            format=self.output_format)
     @staticmethod
-    def iso(date_input):
-        tz = pytz.timezone("Europe/Prague")
-        output= tz.localize(dt.datetime.combine(date_input, dt.time())).astimezone(pytz.utc).strftime('%Y-%m-%dT%H:%M:%SZ')
-        return output
+    def is_parsable(input):
+        from dateutil.parser import parse
+        try:
+            parse(input)
+            return True
+        except:
+            return False
+    @property
+    def output_datetime(self):
+        return self._output_datetime
+
+    @output_datetime.setter
+    def output_datetime(self,output_datetime):
+        self._output_datetime = output_datetime
+        self.output = self.output_datetime if self.output_type == 'datetime' else self.output_datetime.strftime(
+            format=self.output_format)
+
+    def convert_timezone(self):
+        import pytz
+        output_tz = pytz.timezone(self.output_timezone)
+        input_tz=pytz.timezone(self.input_timezone)
+        self.output_datetime= output_tz.localize(self.output_datetime).astimezone(input_tz)
+        return self.output
 
 
-    @staticmethod
-    def today(granularity="datetime"):
-        input = dtd.now()
-        if granularity=="date":
-            output=input.strftime("%Y-%m-%d")
-        elif granularity=="datetime":
-            output=input.strftime("%Y-%m-%dT%H:%M:%S")
-        return output
+    def timedelta(self,relativedelta_object):
+        from dateutil.relativedelta import relativedelta
+        relativedelta_arguments=",".join([f"{element[0]}={element[1]}" for element in relativedelta_object.items()])
+        exec(f"change = relativedelta({relativedelta_arguments})", locals(), globals())
+        self.output_datetime=self.output_datetime+change
+        return self.output
 
-    @staticmethod
-    def today_plus(granularity="datetime",dplus=0,out_format="string"):
-        date_add = (dt.datetime.now() + dt.timedelta(days=dplus))
-        if out_format == "string":
-            output=date_add.strftime("%Y-%m-%dT%H:%M:%S")
-            output = output[:10] if granularity == "date" else output
+    def get_rel_date(self,input):
+        import datetime as dt
+
+        if input == "today":
+            self.output_datetime = dt.datetime.now()
+
+        return self.output
+
+    def infer_function(self,input):
+        if isinstance(input, dict):
+            output = self.timedelta(relativedelta_object=input)
+
+        elif time.is_parsable(input):
+            output = time(input=input,
+                                          output_type=self.output_type,
+                                          output_format=self.output_format,
+                                          input_tz=self.input_timezone,
+                                          output_tz=self.output_timezone).output
         else:
-            output=date_add
-
+            output = self.get_rel_date(input=input)
         return output
 
+    def get_time_range(self,**kwargs):
+        import numpy as np
+        from dateutil.parser import parse
+        range_object = {"bound":{},"range":{}}
+        _range=range_object["range"]
+        range_object["bound"]["names"]=["start", "end"]
+        _defined=np.intersect1d(list(kwargs.keys()),range_object["bound"]["names"])
+        if len(_defined) == 0:
+            raise KeyError("at least one of start , end arguments must be used")
+        elif len(_defined) == 1:
+
+            _unknown=next(iter(set(range_object["bound"]["names"])-set(_defined)))
+            _range[_unknown]=self.output
+            _known=_defined[0]
+            input=kwargs[_known]
+            _range[_known]=self.infer_function(input)
+
+        elif len(_defined)==2:
+            _known=[i for i in _defined if str(kwargs[i]) in "today" or time.is_parsable(kwargs[i])]
+            _unknown=next(iter(set(range_object["bound"]["names"])-set(_known)))
+            for i in _known:
+                _range[i]=self.infer_function(kwargs[i])
+            if _unknown:
+                _range[_unknown]=time(input=_range[_known[0]],
+                                      input_tz=self.input_timezone,
+                                      output_tz=self.output_timezone,
+                                      output_format=self.output_format,
+                                      output_type=self.output_type).infer_function(kwargs[_unknown])
+
+
+        return _range
 
 class pandas:
     @staticmethod
@@ -233,7 +308,7 @@ class local:
 class query:
     @staticmethod
     def get_last_value(column,tablePath):
-        query_string = 'SELECT MAX({}) FROM `{}`'.format(column,tablePath)
+        query_string = 'SELECT MAX({}) FROM {}'.format(column,tablePath)
         return query_string
 
 
